--
-- Jpn2Esp.lua
-- Mapping of Japanese VSQ(X)'s to Spanish phonetics to be used with Bruno / Clara voice banks.
--

--
-- Copyright (C) 2012 Voctro Labs, S.L.
--

--
-- Manifest
--
function manifest()
    myManifest = {
        name          = "Japanese-to-Spanish",
        comment       = "Convert Japanese VSQX for use with Spanish voice bank",
        author        = "Voctro Labs, S.L._168",
        pluginID      = "{F17A37C4-AD2C-433a-B5BC-773F2EACB94C}",
        pluginVersion = "1.0.0.1",
        apiVersion    = "3.0.0.1"
    }
    
    return myManifest
end


--
-- Main
--
function main(processParam, envParam)
	-- -- Process parameters
	-- local beginPosTick = processParam.beginPosTick	-- begin of time range to apply Job to (in ticks)
	-- local endPosTick   = processParam.endPosTick		-- end of time range to apply Job to (in ticks)
	-- local songPosTick  = processParam.songPosTick	-- onset of song (in ticks)

	-- -- Environment parameters
	-- local scriptDir  = envParam.scriptDir	-- script folder
	-- local scriptName = envParam.scriptName	-- script name
	-- local tempDir    = envParam.tempDir		-- temporary folder
	
	local mapping    = {}
	local mapping2   = {} -- exceptions to general rules using two phoneme input string
	local noteEx     = {}
	local noteExList = {}
	local noteCount
	local retCode
	local idx
	
	-- define japanese-to-spanish phoneme mapping
	mapping["a"]	= "a"
	mapping["i"]	= "i"
	mapping["M"]	= "u"	-- poor match; japanese: unrounded, spanish: rounded
	mapping["e"]	= "e"	-- (openness/closedness of japanese and spanish /e/ may not always match, but seem somewhat variable)
	mapping["o"]	= "o"	-- (openness/closedness of japanese and spanish /o/ may not always match, but seem somewhat variable)

	mapping["n"]	= "n"
	mapping["J"]	= "J"	-- japanese: alveolo-palatal nasal, spanish: palatal nasal
	mapping["m"]	= "m"
	mapping["N"]	= "n"	-- japanese: velar nasal (ng), spanish: alveolar nasal (n)
	mapping["m'"]	= "m j"	-- japanese: palatalized, spanish: non-palatalized + j
	mapping2["m' i"] = {"m", "i"} -- japanese: palatalized, spanish: non-palatalized
	-- note: [m j] exists in spanish voice banks, but [j i] doesn't; [j\ i] exists, but [m j\] doesn't
	-- note: /m'/ may sound a little more towards /n/
	mapping["N'"]	= "J"	-- japanese: palatalized velar nasal, spanish: palatal nasal
	mapping["N\\"]	= "n"	-- japanese: uvular nasal, spanish: alveolar nasal
	
	mapping["g"]	= "g"
	mapping["g'"]	= "G j\\"	-- japanese: palatalized, spanish: non-palatalized + j (note: spanish voice banks do not have [g j], but do have [G j])
	mapping["d"]	= "d"
	mapping["d'"]	= "d j\\"	-- japanese: palatalized, spanish: non-palatalized + j
	mapping["b"]	= "b"
	mapping["b'"]	= "b j\\"	-- japanese: palatalized, spanish: non-palatalized + j
	
	mapping["z"]	= "s"	-- poor match; japanese: voiced alveolar fricative, spanish: voiceless alveolar fricative
	mapping["Z"]	= "s"	-- poor match; japanese: voiced postalveolar fricative, spanish: voiceless alveolar fricative
	mapping["h\\"]	= "x"	-- poor match; japanese: voiced glottal fricative, spanish: voiceless velar fricative	
	
	mapping["dz"]	= "D s"	-- poor match (both parts of affricate)
	mapping["dZ"]	= "D s" -- poor match (both parts of affricate)
	-- note: there's no [d s], but there is [D s] (still very poor match)

	mapping["4"]	= "r"	-- (should be approximately same, spanish voice banks use /r/ for x-sampa /4/)
	mapping["4'"]	= "r j\\"	-- (should be approximately same, spanish voice banks use /r/ for x-sampa /4/)
	-- note: japanese /4/ can have variable r-like or l-like sound

	mapping["j"]	= "j"
	mapping["w"]	= "w" -- japanese /w/ sometimes more plosive-like compared to spanish /w/ (can be similar to spanish /B/)

	mapping["k"]	= "k"
	mapping["k'"]	= "k j"	-- japanese: palatalized, spanish: non-palatalized + j
	mapping2["k' i"] = {"k", "i"} -- japanese: palatalized, spanish: non-palatalized
	-- [k j] exists, but [j i] doesn't; [j\ i] exists, but [k j\] doesn't
	mapping["t"]	= "t"
	mapping["t'"]	= "t j\\"	-- japanese: palatalized, spanish: non-palatalized + j
	mapping["p"]	= "p"
	mapping["p'"]	= "p j\\"	-- japanese: palatalized, spanish: non-palatalized + j

	mapping["s"]	= "s"
	mapping["S"]	= "s"	-- poor match; japanese: voiceless postalveolar fricative ("sh"), spanish: voiceless alveolar fricative ("s")
	-- note: sometimes /s j/ may be a better match for /S/ than /s/, but sometimes it sounds too much like /s i/
	mapping["h"]	= "x"	-- poor match; japanese: voiceless glottal fricative, spanish: voiceless velar fricative
	mapping["C"]	= "x j\\"	-- poor match; japanese: voiceless palatal fricative, spanish: voiceless velar fricative + j
	mapping["p\\"]	= "f"	-- japanese: voiceless bilabial fricative ("between /p/ and /f/"), spanish: voiceless labiodental fricative (/p/ = voiceless bilabial plosive)
	mapping["p\\'"]	= "f j\\"	-- japanese: palatalized, spanish: non-palatalized + j
	-- note: in some japanese voice banks /p\/ is more towards /p/ than /f/

	mapping["ts"]	= "t s"	-- japanese: affricate, spanish: consonant cluster
	mapping["tS"]	= "tS"
	-- note: there's no [t s] in spanish voice banks, will be synthesized as [t Sil] [Sil s] (sounds more or less ok, but not very 'affricate-like')
	
	mapping["?"]	= "Sil"	-- poor match; japanese: glottal stop (brief sudden stop), spanish: full stop (long gradual stop)
	
	-- get list of notes
	VSSeekToBeginNote()
	idx = 1
	retCode, noteEx = VSGetNextNoteEx()
	while (retCode == 1) do
		noteExList[idx] = noteEx
		retCode, noteEx = VSGetNextNoteEx()
		idx = idx + 1
	end

	-- check if empty
	noteCount = table.getn(noteExList)
	if (noteCount == 0) then
		VSMessageBox("No notes to process!", 0)
		return 0
	end
	
	-- transform notes
	for idx = 1, noteCount do
		local updNoteEx = {}
		updNoteEx = noteExList[idx]
		
		-- split on whitespace
		local phns = {}
		for token in string.gmatch(updNoteEx.phonemes, "[^%s]+") do
			 table.insert(phns, token)
		end
		
		-- do phoneme-by-phoneme mapping
		for i = 1, table.getn(phns) do
			-- try exceptions first
			local mapped = false
			if i+1 <= table.getn(phns) then
				twophn = phns[i] .. " " .. phns[i+1]
				if mapping2[twophn] then
					phns[i] = mapping2[twophn][1]
					phns[i+1] = mapping2[twophn][2]
					-- (in this case no need to increment i because mapping2[twophn][2] is always "i", so will map to "i" on next iteration)
					mapped = true -- there's no continue in lua
				end
			end
			
			-- general mapping second
			if not mapped then
				if mapping[phns[i]] then
					phns[i] = mapping[phns[i]]
				end
			end
		end
		
		-- join by space
		updNoteEx.phonemes = table.concat(phns, " ");
		
		-- update note
		retCode = VSUpdateNoteEx(updNoteEx);
		if (retCode ~= 1) then
			VSMessageBox("Failed to update note!", 0)
			return 1
		end
	end

	-- Success
	return 0
end
